﻿/**
 * Движок ТММ 3
 * @author Werdn
 * @version 1.0
 * 
	TMM III - simulator of plain mechanisms
	Copyright (C) 2007  Andrey Motrenko (werdn666@yandex.ru)
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ru.narod.werdn666.tmm3;

import flash.Lib;
import flash.MovieClip;
import flash.Stage;
import flash.TextField;
import haxe.Timer;

import ru.narod.werdn666.tmm3.vertices.Vertex;
import ru.narod.werdn666.tmm3.groups.Group;

/**
 * Движок ТММ 3
 */
class Engine 
{
	public var omega:Float;

	private var render(default, null):MovieClip;
	
	private var title:TextField;
	
	private var groups:Array<Group>;
	
	private var tim:Timer;
	
	/**
	 * Заголовок документа
	 */
	public var caption(default, setCaption):String;
	private function setCaption(val:String):String
	{
		title.text = val;
		return val;
	}
	
	/**
	 * Параметр, определяющий видимость заголовка документа
	 */
	public var captionVisible(default, setCaptionVisible):Bool;
	private function setCaptionVisible(val:Bool):Bool
	{
		title._visible = val;
		return val;
	}
	
	public var delay(default, setDelay):Int;
	private function setDelay(val:Int):Int
	{
		if(tim != null)
		{
			stop();
			tim = new Timer(val);
			tim.run = run;
		}
		return val;
	}
	
	/**
	* Конструктор
	* @param	Render	-	клип для рисования
	*/
	public function new(Render:MovieClip) 
	{
		render = Render;
		title = render.createTextField("title", 10, 0, 0, Stage.width, 24);
		title.selectable = false;
		caption = '{unnamed}';
		delay = 10;
		captionVisible = true;
		groups = new Array<Group>();
	}
	
	/**
	 * Движение механизма
	 */
	private function run()
	{
		oneStep();
		paintMech();
	}
	
	/**
	 * Запуск анимации
	 */
	public function start()
	{
		stop();
		tim = new Timer(delay);
		tim.run = run;
	}
	
	/**
	 * Остановка анимации
	 */
	public function stop()
	{
		if(tim != null)
		{
			tim.stop();
			tim = null;
		}
	}
	
	/**
	* Производим движение механизма
	*/
	private function oneStep()
	{
		var i:Int;
		for(i in 0...groups.length)
		{
			groups[i].calc();
		}
	}
	
	/**
	* Рисуем весь механизм
	*/
	private function paintMech()
	{
		render.clear();
		var i:Int;
		for(i in 0...groups.length)
		{
			groups[i].paint(render);
		}
	}
	
	/**
	 * Добавление группы в механизм
	 * @param	g
	 */
	public function addGroup(g:Group)
	{
		groups.push(g);
	}
	
	/**
	 * Очищает механизм
	 */
	public function clear()
	{
		groups = new Array<Group>();
	}
	
}