﻿/**
 * Загрузчик файла данных
 * @author Werdn
 * @version 1.0
 * 
	TMM III - simulator of plain mechanisms
	Copyright (C) 2007  Andrey Motrenko (werdn666@yandex.ru)
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ru.narod.werdn666.tmm3;


import flash.MovieClip;
import flash.Stage;
import haxe.Http;
import ru.narod.werdn666.tmm3.Engine;
import ru.narod.werdn666.tmm3.groups.CrankGroup;
import ru.narod.werdn666.tmm3.groups.Group;
import ru.narod.werdn666.tmm3.groups.TwoLeversGroup;
import ru.narod.werdn666.tmm3.groups.KulisaGroup;
import ru.narod.werdn666.tmm3.groups.CrowlerGroup;
import ru.narod.werdn666.tmm3.groups.SplitterGroup;
import ru.narod.werdn666.tmm3.vertices.CylinderVertex;
import ru.narod.werdn666.tmm3.vertices.GroundVertex;
import ru.narod.werdn666.tmm3.vertices.StoneVertex;
import ru.narod.werdn666.tmm3.vertices.Vertex;

/**
 * Загрузчик файла данных
 */
class MechLoader 
{

	private var eng:Engine;
	private var fileName:String;
	private var loader:MovieClip;
	
	/**
	 * Конструктор. 
	 * @param	engine 	- 	движок, для которого будет произведена загрузка
	 */
	public function new(engine:Engine) 
	{
		eng = engine;		
	}
	
	/**
	 * Загрузка из файла
	 * @param	filename	- имя файла
	 * @param	lmc		-	клип для цепляния прелоадера
	 */
	public function load(filename:String, lmc:MovieClip)
	{
		loader = lmc.attachMovie('library.loader_clip.swf', 'loader', lmc.getNextHighestDepth());
		loader._x = Stage.width / 2;
		loader._y = Stage.height / 2;
		fileName = filename;
		var http:Http = new Http(filename);
		http.onData = xmlLoaded;
		http.onError = xmlNotLoaded;
		http.onStatus = xmlStatus;
		http.request(false);
	}
	
	/**
	 * Обработчик события при загрузке
	 * @param	status	-	состояние загрузки
	 */
	private function xmlStatus(status:Int)
	{
		eng.caption = 'Loading ' + status;
	}
	
	/**
	 * Обработчик успешной загрузки файла данных
	 * @param	data	-	текст XML
	 */
	private function xmlLoaded(data:String)
	{
		eng.clear();
		loader.removeMovieClip();
		//trace('Data ' + data);
		var doc:Xml = Xml.parse(data);
		var tmm:Xml = doc.firstElement();
		var hash:Hash<Vertex> = new Hash();
		//trace(tmm.nodeName);
		if(tmm.nodeName == 'tmm')
		{
			var title:Xml 		= tmm.elementsNamed('title').next();
			if(title != null)
			{
				eng.caption = title.get('name') + ' (' + title.get('author') + ')';
				if(title.exists('delay'))
				{
					eng.delay = Std.parseInt(title.get('delay'));
				}
			}
			var vertices:Xml 	= tmm.elementsNamed('vertices').next();
			var vertex:Xml;
			for(vertex in vertices.elements())
			{
				if(vertex.nodeName == 'vertex')
				{
					try
					{
						var type:String = vertex.get('type');
						var id:String = vertex.get('id');
						var v:Vertex;
						//if(type == '')	type = 'cylinder';
						var x:Int = Std.parseInt(vertex.get('x'));
						var y:Int = Std.parseInt(vertex.get('y'));
						if(id != '')
						{
							switch(type)
							{
								case 'ground':
									v = new GroundVertex(x, y);
								case 'cylinder':
									v = new CylinderVertex(x, y);
								case 'stone':
									v = new StoneVertex(x, y);
								default:
									v = new CylinderVertex(x, y);
							}
							if(vertex.exists('color'))
							{
								v.color = Std.parseInt(vertex.get('color'));
							}
							hash.set(id, v);
						}
					}
					catch(e : Dynamic)
					{
						
					}
				}
			}
			var groups:Xml		= tmm.elementsNamed('groups').next();
			var group:Xml;
			for(group in groups.elements())
			{
				if(group.nodeName == 'group' && group.exists('connect'))
				{
					try
					{
						var type:String = group.get('type');
						var id:String 	= group.get('id');
						var verts:Array<String> = group.get('connect').split(':');
						var g:Group;
						switch(type)
						{
							case 'crank':
								g = new CrankGroup(hash.get(verts[0]), hash.get(verts[1]));
							case 'twolevers':
								g = new TwoLeversGroup(hash.get(verts[0]), hash.get(verts[1]), hash.get(verts[2]));
							case 'kulisa':
								g = new KulisaGroup(hash.get(verts[0]), hash.get(verts[1]), hash.get(verts[2]));
							case 'crowler':
								g = new CrowlerGroup(hash.get(verts[0]), hash.get(verts[1]), hash.get(verts[2]), hash.get(verts[3]));
							case 'splitter':
								g = new SplitterGroup(hash.get(verts[0]), hash.get(verts[1]), hash.get(verts[2]));
							default:
								throw new String("Not defined node - " + type);
						}
						if(group.exists('color'))
						{
							g.color = Std.parseInt(group.get('color'));
						}
						eng.addGroup(g);
					}
					catch (e:Dynamic)
					{
						
					}
				}
			}
		}
	}
	
	/**
	 * Обработчик при неудачной загрузке файла
	 * @param	msg	-	текст сообщения
	 */
	private function xmlNotLoaded(msg:String)
	{
		loader.removeMovieClip();
		//trace(msg);
		eng.caption = msg + ' ' + fileName;
	}
	
}