﻿/**
 * Структурная группа - шатун-ползун
 * @author Werdn
 * @version 1.0
 * 
	TMM III - simulator of plain mechanisms
	Copyright (C) 2007  Andrey Motrenko (werdn666@yandex.ru)
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ru.narod.werdn666.tmm3.groups;

import flash.geom.Point;
import flash.MovieClip;
import ru.narod.werdn666.tmm3.vertices.Vertex;
import ru.narod.werdn666.tmm3.groups.Group;
import ru.narod.werdn666.tmm3.graphics.Graphics;

/**
 * Структурная группа - шатун-ползун
 */
class CrowlerGroup extends Group
{
	public var p1:Vertex;
	public var p2:Vertex;
	public var p3:Vertex;
	public var p4:Vertex;
	
	public var length1:Float;
	public var length2:Float;

	/**
	 * Конструктор
	 * @param	one		-	начало направляющей
	 * @param	two		-	конец направляющей
	 * @param	three	-	начало шатуна
	 * @param	out		-	ползун
	 */
	public function new(one:Vertex, two:Vertex, three:Vertex, out:Vertex) 
	{
		super();
		
		p1 = one;
		p2 = two;
		p3 = three;
		p4 = out;
		
		length1 = Point.distance(p1.center, p2.center);
		length2 = Point.distance(p3.center, p4.center);
	}
	
	/**
	 * Функция расчета текущего положения структурной группы
	 */
	public function calc()
	{
		var PI:Float = Math.PI;
		var a:Float = Graphics.getAngle(p1.center, p2.center);
		
		p4.angle = a;
		/* Расстояние от начала шатуна до второй точки направляющих */
		var l1:Float = Point.distance(p2.center, p3.center);
		/* Расстояние от начала шатуна до первой точки направляющих */
		var l2:Float = Point.distance(p1.center, p3.center);
		
		var h:Float = Graphics.distanceToLine(p3.center, p1.center, p2.center);
		
		var x1:Float = Math.sqrt(l2 * l2 - h * h);
		var x2:Float = Math.sqrt(length2 * length2 - h * h);
		var x3:Float = Math.sqrt(l1 * l1 - h * h);
		var r:Float = 0;
		if(x3 > length1)
		{
			r = x2 - x1;		
		}
		else
		{
			r = x2 + x1;
		}
		if(Math.isNaN(x1))
		{
      r = x2;
    }
		p4.center.x = p1.center.x + r * Math.cos(a);
		p4.center.y = p1.center.y + r * Math.sin(a);
		
		p1.painted = false;
		p2.painted = false;
		p3.painted = false;	
		p4.painted = false;	
	}
	
	/**
	 * Функция отрисовки положения структурной группы
	 * @param	mc	-	место для отрисовки
	 */
	public function paint(mc:MovieClip)
	{
		mc.lineStyle(1, color);
		
		mc.moveTo(p1.center.x, p1.center.y);
		mc.lineTo(p2.center.x, p2.center.y);
		
		mc.moveTo(p3.center.x, p3.center.y);
		mc.lineTo(p4.center.x, p4.center.y);
		
		p1.paint(mc);
		p2.paint(mc);
		p3.paint(mc);
		p4.paint(mc);
	}
	
}