﻿/**
 * Структурная группа - кулиса
 * @author Werdn
 * @version 1.0
 * 
	TMM III - simulator of plain mechanisms
	Copyright (C) 2007  Andrey Motrenko (werdn666@yandex.ru)
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ru.narod.werdn666.tmm3.groups;

import flash.geom.Point;
import flash.MovieClip;
import ru.narod.werdn666.tmm3.vertices.Vertex;
import ru.narod.werdn666.tmm3.groups.Group;
import ru.narod.werdn666.tmm3.graphics.Graphics;

/**
 * Структурная группа - кулиса
 */
class KulisaGroup extends Group
{
	/**
	 * Основные точки кулисной структурной группы
	 */
	public var p1:Vertex;
	public var p2:Vertex;
	public var p3:Vertex;
	
	/**
	 * Длинна коромысла
	 */
	public var length1:Float;
	
	/**
	 * Конструктор для создания кулисы
	 * @param	one	-	неподвижная точка коромысла
	 * @param	two	-	камень
	 * @param	out	-	подвижная точка коромысла
	 */
	public function new(one:Vertex, two:Vertex, out:Vertex) 
	{
		super();
		
		p1 = one;
		p2 = two;
		p3 = out;
		
		length1 = Point.distance(p1.center, p3.center);
		//length2 = Point.distance(p2.center, p3.center);
	}
	
	/**
	 * Функция расчета текущего положения структурной группы
	 */
	public function calc()
	{
		var PI:Float = Math.PI;
		var a:Float = Graphics.getAngle(p1.center, p2.center);
		
		p2.angle = a;
		
		p3.center.x = p1.center.x + length1 * Math.cos(a);
		p3.center.y = p1.center.y + length1 * Math.sin(a);
		
		p1.painted = false;
		p2.painted = false;
		p3.painted = false;	
	}
	
	/**
	 * Функция отрисовки положения структурной группы
	 * @param	mc	-	место для отрисовки
	 */
	public function paint(mc:MovieClip)
	{
		mc.lineStyle(1, color);
		
		mc.moveTo(p1.center.x, p1.center.y);
		mc.lineTo(p3.center.x, p3.center.y);
		
		p1.paint(mc);
		p2.paint(mc);
		p3.paint(mc);		
	}
	
}