﻿/**
 * Разделитель - нужен для ускорения построения разбиения звена на части, без использования треугольников
 * @author Werdn
 * @version 1.0
 * 
	TMM III - simulator of plain mechanisms
	Copyright (C) 2007  Andrey Motrenko (werdn666@yandex.ru)
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ru.narod.werdn666.tmm3.groups;

import flash.geom.Point;
import flash.MovieClip;
import ru.narod.werdn666.tmm3.vertices.Vertex;
import ru.narod.werdn666.tmm3.groups.Group;
import ru.narod.werdn666.tmm3.graphics.Graphics;

/**
 * Разделитель - нужен для ускорения построения разбиения звена на части, без использования треугольников
 */
class SplitterGroup extends Group
{
	public var p1:Vertex;
	public var p2:Vertex;
	public var p3:Vertex;
	
	public var length1:Float;
	public var length2:Float;

	/**
	 * Конструктор разделителя
	 * @param	one	-	начало звена для разделения
	 * @param	two	-	конец звена для разделения
	 * @param	out	-	точка разделения
	 */
	public function new(one:Vertex, two:Vertex, out:Vertex) 
	{
		super();
		
		p1 = one;
		p2 = two;
		p3 = out;
		
		length1 = Point.distance(p1.center, p2.center);
		length2 = Point.distance(p1.center, p3.center);
	}
	
	/**
	 * Функция расчета текущего положения структурной группы
	 */
	public function calc()
	{
		var sinA:Float = (p2.center.x - p1.center.x) / length1;
		var cosA:Float = (p2.center.y - p1.center.y) / length1;
		
		p3.center.x = p1.center.x + length2 * sinA;
		p3.center.y = p1.center.y + length2 * cosA;
	}
	
	/**
	 * Функция отрисовки положения структурной группы
	 * @param	mc	-	место для отрисовки
	 */
	public function paint(mc:MovieClip)
	{
		mc.lineStyle(1, color);
		
		mc.moveTo(p1.center.x, p1.center.y);
		mc.lineTo(p3.center.x, p3.center.y);
		mc.lineTo(p2.center.x, p2.center.y);
		
		mc.moveTo(p3.center.x, p3.center.y);
		mc.beginFill(color);
		Graphics.paintCircle(mc, p3.center.x, p3.center.y, 3);
		mc.endFill();
		mc.moveTo(p3.center.x, p3.center.y);
			
		p1.paint(mc);
		p2.paint(mc);
		p3.paint(mc);		
	}
	
}